# Generated by Django 1.10.7 on 2017-07-25 16:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0018_log_snapshot_txt"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="snapshot_txt",
            field=models.TextField(blank=True, verbose_name="Snapshot Txt"),
        ),
    ]
