# Generated by Django 1.10.5 on 2017-01-18 17:42
# Simplified by Juanmi Taboada on 2017-14-02 11:12

from typing import Any, List

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0006_auto_20160825_0838"),
    ]

    operations: List[Any] = []
